﻿Imports System.IO
Imports System.Windows.Printing
Imports System.Text

Partial Public Class MainPage
    Inherits UserControl

    Private Shared ReadOnly desiredMargin As New Thickness(48)
    Private leftOverLine As String
    Private streamReader As StreamReader
    Private fileInfo As FileInfo

    Public Sub New()
        InitializeComponent()
    End Sub

    Private Sub OnLoadButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim dlg As New OpenFileDialog()

        If dlg.ShowDialog().Value = True Then
            fileInfo = dlg.File
            printButton.IsEnabled = True
        End If
    End Sub

    Private Sub OnPrintButtonClick(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim printDocument As New PrintDocument()
        AddHandler printDocument.BeginPrint, Sub(printDoc As Object, beginArgs As BeginPrintEventArgs) streamReader = fileInfo.OpenText()
        AddHandler printDocument.EndPrint, Sub(printDoc As Object, endArgs As EndPrintEventArgs) streamReader.Dispose()
        AddHandler printDocument.PrintPage, AddressOf OnPrintDocumentPrintPage
        printDocument.Print(fileInfo.Name)
    End Sub

    Private Sub OnPrintDocumentPrintPage(ByVal sender As Object, ByVal args As PrintPageEventArgs)
        Dim border As Border = New Border With {.Padding = New Thickness(Math.Max(0, desiredMargin.Left - args.PageMargins.Left), Math.Max(0, desiredMargin.Top - args.PageMargins.Top), Math.Max(0, desiredMargin.Right - args.PageMargins.Right), Math.Max(0, desiredMargin.Bottom - args.PageMargins.Bottom))}

        Dim stkpnl As New StackPanel()
        border.Child = stkpnl
        Dim line As String = leftOverLine

        line = streamReader.ReadLine()
        Do While (leftOverLine IsNot Nothing) OrElse (line IsNot Nothing)
            leftOverLine = Nothing

            ' Check for blank lines; print them with a space
            If line.Length = 0 Then
                line = " "
            End If

            Dim txtblk As TextBlock = New TextBlock With {.Text = line, .TextWrapping = TextWrapping.Wrap}

            stkpnl.Children.Add(txtblk)
            border.Measure(New Size(args.PrintableArea.Width, Double.PositiveInfinity))

            ' Check if the page is now too tall
            If border.DesiredSize.Height > args.PrintableArea.Height Then
                ' If so, remove some text and save it for later
                leftOverLine = RemoveText(border, txtblk, args.PrintableArea)
                Exit Do
            End If
            line = streamReader.ReadLine()
        Loop

        If leftOverLine Is Nothing Then
            leftOverLine = streamReader.ReadLine()
        End If

        args.PageVisual = border
        args.HasMorePages = leftOverLine IsNot Nothing
    End Sub

    Private Function RemoveText(ByVal border As Border, ByVal txtblk As TextBlock, ByVal printableArea As Size) As String
        Dim leftOverText As New StringBuilder()

        Do
            Dim index As Integer = txtblk.Text.LastIndexOf(" "c)

            If index = -1 Then
                index = 0
            End If

            leftOverText.Insert(0, txtblk.Text.Substring(index))
            txtblk.Text = txtblk.Text.Substring(0, index)
            border.Measure(New Size(printableArea.Width, Double.PositiveInfinity))

            If index = 0 Then
                Exit Do
            End If
        Loop While border.DesiredSize.Height > printableArea.Height

        Return leftOverText.ToString().TrimStart(" "c)
    End Function

End Class