﻿Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports Microsoft.Phone.Controls

Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private Shared ReadOnly blackBrush As Brush = New SolidColorBrush(Colors.Black)
    Private paragraphs As New List(Of String)()
    Private bookInfo As BookInfo = (TryCast(Application.Current, App)).BookInfoSettings

    Public Sub New()
        InitializeComponent()
        PreprocessBook("Books/158.txt")
        AddHandler Loaded, AddressOf OnMainPageLoaded
    End Sub

    Private Sub PreprocessBook(ByVal resourceName As String)
        Using stream As Stream = Application.GetResourceStream(New Uri(resourceName, UriKind.Relative)).Stream
            Using reader As New StreamReader(stream)
                Dim line As String
                Dim paragraph As New StringBuilder()

                line = reader.ReadLine()
                Do While Nothing IsNot line
                    ' Check for a possible paragraph termination
                    If line.Length = 0 OrElse line.Chars(0) = " "c Then
                        If paragraph.Length > 0 Then
                            paragraphs.Add(paragraph.ToString())
                            paragraph.Remove(0, paragraph.Length)
                        End If

                        ' Add the line anyway
                        paragraphs.Add(line)
                    Else
                        ' Insert a space between appended lines assembled into paragraph
                        If paragraph.Length > 0 Then
                            paragraph.Append(" ")
                        End If

                        paragraph.Append(line)
                    End If
                    line = reader.ReadLine()
                Loop

                ' Add the last paragraph if there's one in progress
                If paragraph.Length > 0 Then
                    paragraphs.Add(paragraph.ToString())
                End If
            End Using
        End Using
    End Sub

    Private Sub OnMainPageLoaded(ByVal sender As Object, ByVal args As RoutedEventArgs)
        GoToPage(bookInfo.PageNumber)
    End Sub

    Private Sub OnGestureListenerTap(ByVal sender As Object, ByVal args As GestureEventArgs)
        NextPage()
    End Sub

    Private Sub OnGestureListenerFlick(ByVal sender As Object, ByVal args As FlickGestureEventArgs)
        If args.Angle < 45 OrElse args.Angle > 315 Then
            PreviousPage()

        ElseIf args.Angle > 135 AndAlso args.Angle < 225 Then
            NextPage()
        End If
    End Sub

    Private Sub NextPage()
        ' Check for end of book
        If bookInfo.PageNumber + 2 = bookInfo.Pages.Count AndAlso bookInfo.Pages(bookInfo.PageNumber + 1).ParagraphIndex = -1 Then
            Return
        End If

        GoToPage(bookInfo.PageNumber + 1)
    End Sub

    Private Sub PreviousPage()
        If bookInfo.PageNumber > 0 Then
            GoToPage(bookInfo.PageNumber - 1)
        End If
    End Sub

    Private Sub GoToPage(ByVal pageNumber As Integer)
        pageNumber = Math.Max(0, pageNumber)

        ' Paginate all pages up to the destination page
        Do While bookInfo.Pages.Count <= pageNumber AndAlso bookInfo.Pages(bookInfo.Pages.Count - 1).ParagraphIndex <> -1
            Dim pageInfo As PageInfo = bookInfo.Pages(bookInfo.Pages.Count - 1)
            Dim paragraphIndex As Integer = pageInfo.ParagraphIndex
            Dim characterIndex As Integer = pageInfo.CharacterIndex

            Paginate(paragraphIndex, characterIndex)
            pageInfo = New PageInfo With {.ParagraphIndex = paragraphIndex, .CharacterIndex = characterIndex}
            bookInfo.Pages.Add(pageInfo)
        Loop

        ' Check that the page number is still within bounds
        If pageNumber > bookInfo.Pages.Count - 1 OrElse bookInfo.Pages(pageNumber).ParagraphIndex = -1 Then
            pageNumber = bookInfo.Pages.Count - 2
        End If

        ' Now paginate the destination page
        Dim pageInfo2 As PageInfo = bookInfo.Pages(pageNumber)
        Dim paragraphIndex2 As Integer = pageInfo2.ParagraphIndex
        Dim characterIndex2 As Integer = pageInfo2.CharacterIndex
        Paginate(paragraphIndex2, characterIndex2)

        ' Add PageInfo for next page if it's not already there 
        If pageNumber = bookInfo.Pages.Count - 1 Then
            pageInfo2 = New PageInfo With {.ParagraphIndex = paragraphIndex2, .CharacterIndex = characterIndex2}
            bookInfo.Pages.Add(pageInfo2)
        End If
        bookInfo.PageNumber = pageNumber
        pageNumberRun.Text = (bookInfo.PageNumber + 1).ToString()
    End Sub

    Private Sub Paginate(ByRef paragraphIndex As Integer, ByRef characterIndex As Integer)
        StackPanel.Children.Clear()

        Do While paragraphIndex < paragraphs.Count
            ' Skip if a blank line is the first paragraph on a page
            If StackPanel.Children.Count = 0 AndAlso characterIndex = 0 AndAlso paragraphs(paragraphIndex).Length = 0 Then
                paragraphIndex += 1
                Continue Do
            End If

            Dim txtblk As TextBlock = New TextBlock With {.Text = paragraphs(paragraphIndex).Substring(characterIndex), .TextWrapping = TextWrapping.Wrap, .Foreground = blackBrush}

            ' Check for a blank line between paragraphs
            If txtblk.Text.Length = 0 Then
                txtblk.Text = " "
            End If

            StackPanel.Children.Add(txtblk)
            StackPanel.Measure(New Size(pageHost.ActualWidth, Double.PositiveInfinity))

            ' Check if the StackPanel fits in the available height
            If StackPanel.DesiredSize.Height > pageHost.ActualHeight Then
                ' Strip words off the end until it fits
                Do
                    Dim index As Integer = txtblk.Text.LastIndexOf(" "c)

                    If index = -1 Then
                        index = 0
                    End If

                    txtblk.Text = txtblk.Text.Substring(0, index)
                    StackPanel.Measure(New Size(pageHost.ActualWidth, Double.PositiveInfinity))

                    If index = 0 Then
                        Exit Do
                    End If
                Loop While StackPanel.DesiredSize.Height > pageHost.ActualHeight

                characterIndex += txtblk.Text.Length

                ' Skip over the space
                If txtblk.Text.Length > 0 Then
                    characterIndex += 1
                End If

                Exit Do
            End If
            paragraphIndex += 1
            characterIndex = 0
        Loop
        ' Flag the page beyond the last
        If paragraphIndex = paragraphs.Count Then
            paragraphIndex = -1
        End If
    End Sub
End Class